import base64
import hashlib
import secrets
from typing import Literal, override

from ab_core.pkce.schema.pkce_method import PKCEMethod

from .base import PKCEGeneratorBase


class S256PKCE(PKCEGeneratorBase):
    """Generate PKCE code verifier & code challenge."""

    method: Literal[PKCEMethod.S256] = PKCEMethod.S256

    @staticmethod
    @override
    def generate_code_verifier(length: int = 64) -> str:
        return secrets.token_urlsafe(length)[:128]  # max PKCE length = 128

    @staticmethod
    @override
    def generate_code_challenge(verifier: str) -> str:
        digest = hashlib.sha256(verifier.encode()).digest()
        return base64.urlsafe_b64encode(digest).decode().rstrip("=")



import secrets
from typing import Literal, override

from ab_core.pkce.schema.pkce_method import PKCEMethod

from .base import PKCEGeneratorBase


class PlainPKCE(PKCEGeneratorBase):
    """Legacy PKCE implementation using `plain` method (verifier == challenge)."""

    method: Literal[PKCEMethod.PLAIN] = PKCEMethod.PLAIN

    @staticmethod
    @override
    def build_verifier(length: int = 64) -> str:
        return secrets.token_urlsafe(length)[:128]

    @staticmethod
    @override
    def build_challenge(verifier: str) -> str:
        return verifier



from typing import Annotated, Union

from pydantic import Discriminator

from .plain import PlainPKCE
from .s256 import S256PKCE
from .template import TemplatePKCE

PKCE = Annotated[
    Union[PlainPKCE, S256PKCE, TemplatePKCE],
    Discriminator("method"),
]



from typing import Literal, override

from ab_core.pkce.schema.pkce_method import PKCEMethod

from .base import PKCEGeneratorBase


class TemplatePKCE(PKCEGeneratorBase):
    """Generate PKCE code verifier & code challenge."""

    method: Literal[PKCEMethod.TEMPLATE] = PKCEMethod.TEMPLATE

    @staticmethod
    @override
    def generate_code_verifier(length: int = 64) -> str:
        raise NotImplementedError()

    @staticmethod
    @override
    def generate_code_challenge(verifier: str) -> str:
        raise NotImplementedError()



from abc import ABC, abstractmethod

from pydantic import BaseModel, model_validator


class PKCEGeneratorBase(BaseModel, ABC):
    verifier: str = None
    challenge: str = None

    @model_validator(mode="after")
    def populate_fields(self):
        if self.verifier is None:
            self.verifier = self.generate_code_verifier()
        if self.challenge is None:
            self.challenge = self.generate_code_challenge(self.verifier)
        return self

    @staticmethod
    @abstractmethod
    def generate_code_verifier(length: int = 64) -> str: ...

    @staticmethod
    @abstractmethod
    def generate_code_challenge(verifier: str) -> str: ...
